; ---------------------------------------------------------------------------
; AD HOC Queries in Paradox
;      Programming user defined query options into your applications
;
; Dan Ehrmann - Kallista, Inc.
;
; LIBRARY SCRIPT - Creates Procedures used by ADHOC.SC
;
; These code examples may freely by used in your own applications, at
; your own risk.  Dan Ehrmann and Kallista, Inc. specifically exclude
; any liability for your use of these techniques and examples.
; ---------------------------------------------------------------------------

LIBNAME = "ADHOC"
createlib LIBNAME

clear
@ 0,0 ?? "Creating Ad Hoc Procedures Library"
@ 1,0

; ---------------------------------------------------------------------------
; QUERY BY IMAGE
;
; Allows the user to query a table and specify any conditions.
; Produces an ANSWER table with the same structure as the original
; so that reports can be copied over.
;
; This program will NOT work in Paradox RunTime, where WAIT on a
; query image is one of the few Paradox features not supported.
; ---------------------------------------------------------------------------
proc QUERY_BY_IMAGE()
   private PROCNAME

   PROCNAME = "QUERY_BY_IMAGE"

   menu {ask} {CLAIMS}
   check                       ; checks all fields in the table
   while (true)
      wait table
         prompt "Specify criteria for this query                  [F2] Process       [Esc] Cancel"
      until "F2", "Esc", "DOS", "DOSBig", "Rotate"
                                           ; we need to disable some keys
      switch
         case retval = "Esc" :
            if isfieldview() then          ; cannot clearimage in FieldView
               enter
            endif
            clearimage
            clear
            return

         case retval = "F2" :
            if isfieldview() then          ; cannot process in FieldView
               enter
            endif
            do_it!

            ; indicates a query which is incorrectly formatted

            if window() <> "" then
               message window(), "; press a key"
               retval = getchar()
               loop
            endif

            if isempty("ANSWER") then
               clearimage
               message "No entries satisfy your criteria; press a key"
               retval = getchar()
            else
               clearimage
               message nrecords("ANSWER"), " entries satisfy your criteria; press a key"
               retval = getchar()

               ; the following code is used for this demonstration only.
               ; in a real application, we would jump to a procedure
               ; for selecting and outputting reports
               ;
               ; PROCESS_REPORTS()

               view "ANSWER"
               wait table
                  prompt "[F2] when you have finished viewing the results of your query"
               until "F2"
               clearimage
               clear

            endif

         otherwise :
            beep

      endswitch

   endwhile

endproc

writelib LIBNAME QUERY_BY_IMAGE
release procs QUERY_BY_IMAGE
?? "."




; ---------------------------------------------------------------------------
; QUERY BY FORM
;
; These procedures allow users to specify query criteria into an analogue
; of the data table.  The script "reads" these criteria, and interprets
; then into the query which is created behind the scenes.
; ---------------------------------------------------------------------------
proc QUERY_BY_FORM()
   private PROCNAME,                 ; procedure name
           KCHOICE,                  ; menu choice
           RETURN_FIELD              ; field to return to after a process

   PROCNAME = "QUERY_BY_FORM"

   coedit "CLAIMREP"
   del
   pickform "F"

   while (true)
      wait record
         prompt "Specify criteria for this query.           [F2] Process             [Esc] Cancel",
                "                                           [F5] Reset Conditions                "
      until "F2", "F5", "Esc", "DOS", "DOSBig", "Rotate"

      switch
         case retval = "F5" :
            showmenu
               "No"  : "Are you sure you want to reset query criteria?",
               "Yes" : "Are you sure you want to reset query criteria?"
            to KCHOICE
            if KCHOICE = "Yes" then
               del
               ctrlhome
            endif

         case retval = "F2" :
            do_it!
            RETURN_FIELD = field()           ; save field position

            ; process the criteria, and call the reports routine

            PROCESS_QUERY_CRITERIA()

            coedit "CLAIMREP"
            pickform "F"
            moveto field RETURN_FIELD        ; restore field position

         case retval = "Esc" :
            showmenu
               "No"  : "Are you sure you want to cancel and return to the menu?",
               "Yes" : "Are you sure you want to cancel and return to the menu?"
            to KCHOICE
            if KCHOICE = "Yes" then
               do_it!
               clearimage
               clear
               return
            endif

         otherwise :
            beep

      endswitch

   endwhile

endproc

writelib LIBNAME QUERY_BY_FORM
release procs QUERY_BY_FORM
?? "."



; ---------------------------------------------------------------------------
; Reads the specified criteria, paints and processes the query
; used in the Query-By-Form routine
; ---------------------------------------------------------------------------
proc PROCESS_QUERY_CRITERIA()
   private PROCNAME,                      ; procedure name
           REPCRITS,                      ; report criteria
           ISCRITS                        ; are criteria actually specified

   PROCNAME = "PROCESS_QUERY_CRITERIA"

   ; this next piece of code is necessary in case the CLAIMREP table
   ; is not empty, but there are actually no criteria specified.  This
   ; is possible under some editing circumstances.

   ISCRITS = False
   if isempty("CLAIMREP") = False then
      copytoarray REPCRITS
      clearimage
      for A from 2 to arraysize(REPCRITS)
         if isblank(REPCRITS[A]) = False then
            ISCRITS = True
            quitloop
         endif
      endfor
      if ISCRITS = False then
         message "No criteria to process; press a key"
         retval = getchar()
         return False
      endif
   else
      clearimage
      message "No criteria to process; press a key"
      retval = getchar()
      return False
   endif

   menu {ask} {CLAIMS}
   check

   FILL_CRITERIA (  2, "Claim ID"      , False )
   FILL_CRITERIA (  3, "Split Claim"   , False )
   FILL_CRITERIA (  4, "Claimant"      , True  )
   FILL_CRITERIA (  5, "Sort Key"      , False )
   FILL_CRITERIA (  6, "City"          , True  )
   FILL_CRITERIA (  7, "State"         , False )
   FILL_CRITERIA (  8, "Class Code"    , False )
   FILL_CRITERIA (  9, "Status Code"   , False )
   FILL_CRITERIA ( 10, "Filing Date"   , False )
   FILL_CRITERIA ( 11, "Filing Amount" , False )
   FILL_CRITERIA ( 12, "Settled Date"  , False )
   FILL_CRITERIA ( 13, "Settled Amount", False )
   FILL_CRITERIA ( 14, "Order Date"    , False )

   Do_It!
   if window() <> "" then
      message window(), "; press a key"
      retval = getchar()
      clearimage
      return False
   endif

   if isempty("ANSWER") then
      message "No claims satisfy your criteria; press a key"
      retval = getchar()
      clearall
      return False
   endif

   message nrecords("ANSWER"), " claims satisfy your criteria; press a key"
   retval = getchar()
   clearall

   ; the following code is used for this demonstration only.
   ; in a real application, we would jump to a procedure
   ; for selecting and outputting reports
   ;
   ; PROCESS_REPORTS()

   view "ANSWER"
   firstshow
   wait table
      prompt "[F2] when you have finished viewing the results of your query"
   until "F2"
   clearimage
   clear
   return True

endproc

writelib LIBNAME PROCESS_QUERY_CRITERIA
release procs PROCESS_QUERY_CRITERIA
?? "."



; ---------------------------------------------------------------------------
; Fill criteria into fields if not blank
; ---------------------------------------------------------------------------
proc FILL_CRITERIA( ELEMENTNO ,          ; which array element
                    WHICHFIELD ,         ; field to move to
                    DOWILDCARDS          ; are embedded special characters possible
                  )

   if isblank(REPCRITS [ ELEMENTNO ] ) then
      return
   else
      moveto field WHICHFIELD
      if DOWILDCARDS = True then
         typein "..\"" + REPCRITS [ ELEMENTNO ] + "\".."
      else
         typein REPCRITS [  ELEMENTNO ]
      endif
   endif

endproc

writelib LIBNAME FILL_CRITERIA
release procs FILL_CRITERIA
?? "."



; -------------------------------------------------------------------
; QUERY BY MENU
;
; This set of procedures controls the query definition process using
; a series of standard menus.  Different types of data are entered,
; and text describing the criteria is maintained on the menu
; description line.
; -------------------------------------------------------------------
proc QUERY_BY_MENU ()
   private KCHOICE,                ; menu choice
           PROCNAME,               ; procedure name
           REPCRITS,               ; array of report criteria text strings
           HOWBIG,                 ; number of array elements
           X                       ; a counter for RESET

   PROCNAME = "QUERY_BY_MENU"
   HOWBIG   = 13                   ; number of elements to maintain

   menu {ask} {CLAIMS}             ; load up the query image
      check

   array REPCRITS [ HOWBIG ]       ; initialize array elements
   for X from 1 to  HOWBIG
      REPCRITS[X] = "All values used"
   endfor

   KCHOICE = "Claim"
   while (true)
      showmenu
         "Claim"     : "Limit the query by Claim ID, Split Claim and Claimant Name",
         "Location"  : "Limit the query by City and State",
         "Status"    : "Limit the query by Status and Class Codes",
         "Dates"     : "Limit the query by Filing Date, Settled Date and Order Date",
         "Amounts"   : "Limit the query by Filing Amount and Settled Amount",
         "GO"        : "Process the query using the limiting criteria you have specified",
         "Reset"     : "Remove all limiting criteria",
         "Quit"      : "Return to the previous menu"
      default KCHOICE
      to KCHOICE

      switch
         case KCHOICE = "Claim" :
            CLAIM_CRITERIA()

         case KCHOICE = "Location" :
            LOCATION_CRITERIA()

         case KCHOICE = "Status" :
            STATUS_CRITERIA()

         case KCHOICE = "Dates" :
            DATES_CRITERIA()

         case KCHOICE = "Amounts" :
            AMOUNTS_CRITERIA()

         case KCHOICE = "GO" :
            Do_It!
            if window() <> "" then
               message window(), "; press a key"
               retval = getchar()
               loop
            endif

            if isempty("ANSWER") then
               clearimage
               message "No entries satisfy your criteria; press a key"
               retval = getchar()
            else
               clearimage
               message nrecords("ANSWER"), " entries satisfy your criteria; press a key"
               retval = getchar()

               ; the following code is used for this demonstration only.
               ; in a real application, we would jump to a procedure
               ; for selecting and outputting reports
               ;
               ; PROCESS_REPORTS()

               view "ANSWER"
               firstshow
               wait table
                  prompt "[F2] when you have finished viewing the results of your query"
               until "F2"
               clearimage
               clear

            endif

         case KCHOICE = "Reset" :
            showmenu
               "No"  : "Are you sure you want to remove all limiting criteria?",
               "Yes" : "Are you sure you want to remove all limiting criteria?"
            to KCHOICE2
            if KCHOICE2 = "Yes" then
               for X from 1 to HOWBIG
                  REPCRITS[X] = "All values used"
               endfor
            endif

         otherwise :
            clearimage
            return

      endswitch

   endwhile

endproc

writelib LIBNAME QUERY_BY_MENU
release procs QUERY_BY_MENU



; ---------------------------------------------------------------------------
; Specify limiting criteria for claim, Split claim and Claimant name
; ---------------------------------------------------------------------------
proc CLAIM_CRITERIA()
   private KCHOICE,                    ; menu choice
           WHICHVAL,                   ; value to be assigned to field
           PROCNAME                    ; procedure name

   PROCNAME = "CLAIM_CRITERIA"

   KCHOICE = "ClaimID"
   while (true)
      showmenu
         "ClaimID"        : "Specify Claim ID.  Now: "+REPCRITS[1],
         "SplitClaim"     : "Specify Split Claim.  Now: "+REPCRITS[2],
         "NameOfClaimant" : "Specify Claimant's name.  Now: "+REPCRITS[3],
         "KeySort"        : "Specify Sort Key value.  Now: "+REPCRITS[4],
         "Quit"           : "Return to the previous menu"
      default KCHOICE
      to KCHOICE

      switch
         case KCHOICE = "ClaimID" :
            if REPCRITS[1] <> "All values used" then
               WHICHVAL = REPCRITS[1]
            else
               WHICHVAL = ""
            endif
            WHICHVAL = ENTER_VALUE ("Claim ID", "ID", "&###", WHICHVAL)
            if WHICHVAL <> False then
               moveto [Claim ID]
               ctrlbackspace
               if isblank(WHICHVAL) then
                  REPCRITS[1] = "All values used"
               else
                  typein WHICHVAL
                  REPCRITS[1] = WHICHVAL
               endif
            endif

         case KCHOICE = "SplitClaim" :
            SPLIT_CLAIMS()

         case KCHOICE = "NameOfClaimant" :
            if REPCRITS[3] <> "All values used" then
               WHICHVAL = REPCRITS[3]
            else
               WHICHVAL = ""
            endif
            WHICHVAL = ENTER_VALUE ("Claimant's Name", "Name", "*@", WHICHVAL )
            if WHICHVAL <> False then
               moveto [Claimant]
               ctrlbackspace
               if isblank(WHICHVAL) then
                  REPCRITS[3] = "All values used"
               else
                  typein WHICHVAL
                  REPCRITS[3] = WHICHVAL
               endif
            endif

         case KCHOICE = "KeySort" :
            if REPCRITS[4] <> "All values used" then
               WHICHVAL = REPCRITS[4]
            else
               WHICHVAL = ""
            endif
            WHICHVAL = ENTER_VALUE ("SortKey value", "SortKey", "*!", WHICHVAL )
            if WHICHVAL <> False then
               moveto [Sort Key]
               ctrlbackspace
               if isblank(WHICHVAL) then
                  REPCRITS[4] = "All values used"
               else
                  typein WHICHVAL
                  REPCRITS[4] = WHICHVAL
               endif
            endif

         otherwise :
            return

      endswitch
   endwhile

endproc

writelib LIBNAME CLAIM_CRITERIA
release procs CLAIM_CRITERIA
?? "."



; -------------------------------------------------------------------
; Specify Split limits
; -------------------------------------------------------------------
proc SPLIT_CLAIMS()
   private KCHOICE,
           PROCNAME

   PROCNAME = "SPLIT_CLAIMS"

   moveto [Split Claim]
   if isblank([]) then
      KCHOICE = "AllClaims"
   else
      if [] = "blank" then
         KCHOICE = "NonSplit"
      else
         KCHOICE = "Split"
      endif
   endif

   showmenu
      "Split"     : "Include split claims only",
      "NonSplit"  : "Include only claims which are not split",
      "AllClaims" : "Include all claims in the output",
      "Quit"      : "Return to the previous menu"
   default KCHOICE
   to KCHOICE
   switch
      case KCHOICE = "Split" :
         ctrlbackspace
         typein "not blank"
         REPCRITS[2] = "Split Claims only"

      case KCHOICE = "NonSplit" :
         ctrlbackspace
         typein "blank"
         REPCRITS[2] = "Non Split Claims only"

      case KCHOICE = "AllClaims" :
         ctrlbackspace
         REPCRITS[2] = "All values used"

   endswitch

endproc

writelib LIBNAME SPLIT_CLAIMS
release procs SPLIT_CLAIMS
?? "."



; ---------------------------------------------------------------------------
; Specify limiting criteria for city and state
; ---------------------------------------------------------------------------
proc LOCATION_CRITERIA()
   private KCHOICE,
           PROCNAME

   PROCNAME = "LOCATION_CRITERIA"

   KCHOICE = "City"
   while (true)
      showmenu
         "City"  : "Specify City limits.  Now: "+REPCRITS[5],
         "State" : "Specify State limits.  Now: "+REPCRITS[6],
         "Quit"  : "Return to the previous menu"
      default KCHOICE
      to KCHOICE

      switch
         case KCHOICE = "City" :
            if REPCRITS[5] <> "All values used" then
               WHICHVAL = REPCRITS[5]
            else
               WHICHVAL = ""
            endif
            WHICHVAL = ENTER_VALUE ("City", "City", "*@", WHICHVAL )
            if WHICHVAL <> False then
               moveto [City]
               ctrlbackspace
               if isblank(WHICHVAL) then
                  REPCRITS[5] = "All values used"
               else
                  typein WHICHVAL
                  REPCRITS[5] = WHICHVAL
               endif
            endif

         case KCHOICE = "State" :
            if REPCRITS[6] <> "All values used" then
               WHICHVAL = REPCRITS[6]
            else
               WHICHVAL = ""
            endif
            WHICHVAL = ENTER_VALUE ("State", "State", "*@", WHICHVAL )
            if WHICHVAL <> False then
               moveto [State]
               ctrlbackspace
               if isblank(WHICHVAL) then
                  REPCRITS[6] = "All values used"
               else
                  typein WHICHVAL
                  REPCRITS[6] = WHICHVAL
               endif
            endif

         otherwise :
            return

      endswitch
   endwhile

endproc

writelib LIBNAME LOCATION_CRITERIA
release procs LOCATION_CRITERIA
?? "."


; ---------------------------------------------------------------------------
; Specify limiting criteria for Class and Status Codes
; ---------------------------------------------------------------------------
proc STATUS_CRITERIA()
   private KCHOICE,
           PROCNAME

   PROCNAME = "STATUS_CRITERIA"

   KCHOICE = "ClassCode"
   while (true)
      showmenu
         "ClassCode"  : "Specify Class Code limits.  Now: "+REPCRITS[7],
         "StatusCode" : "Specify Status Code limits.  Now: "+REPCRITS[8],
         "Quit"       : "Return to the previous menu"
      default KCHOICE
      to KCHOICE

      switch
         case KCHOICE = "ClassCode" :
            LOOKUP_TABLE ( "CLASS", "Class Code", "Class Code", 7 )

         case KCHOICE = "StatusCode" :
            LOOKUP_TABLE ( "STATUS", "Status Code", "Status Code", 8 )

         otherwise :
            return

      endswitch
   endwhile

endproc

writelib LIBNAME STATUS_CRITERIA
release procs STATUS_CRITERIA
?? "."



; ---------------------------------------------------------------------------
; Specify limiting criteria for Filing date, Settled date and Order date
; ---------------------------------------------------------------------------
proc DATES_CRITERIA()
   private KCHOICE,
           PROCNAME

   PROCNAME = "DATES_CRITERIA"

   KCHOICE = "FilingDate"
   while (true)
      showmenu
         "FilingDate"  : "Specify Filing Date limits.  Now: "+REPCRITS[9],
         "SettledDate" : "Specify Settled Date limits.  Now: "+REPCRITS[11],
         "OrderDate"   : "Specify Order Date limits.  Now: "+REPCRITS[13],
         "Quit"        : "Return to the previous menu"
      default KCHOICE
      to KCHOICE

      switch
         case KCHOICE = "FilingDate" :
            DATE_OR_NUMBER ( "Date", "Filing Date", "Filing Date", 9 )

         case KCHOICE = "SettledDate" :
            DATE_OR_NUMBER ( "Date", "Settled Date", "Settled Date", 11 )

         case KCHOICE = "OrderDate" :
            DATE_OR_NUMBER ( "Date", "Order Date", "Order Date", 13 )

         otherwise :
            return

      endswitch
   endwhile

endproc

writelib LIBNAME DATES_CRITERIA
release procs DATES_CRITERIA
?? "."



; ---------------------------------------------------------------------------
; Specify limiting criteria for Filing amount and Settled amount
; ---------------------------------------------------------------------------
proc AMOUNTS_CRITERIA()
   private KCHOICE,
           PROCNAME

   PROCNAME = "AMOUNTS_CRITERIA"

   KCHOICE = "FilingAmount"
   while (true)
      showmenu
         "FilingAmount"  : "Specify Filing Amount limits.  Now: "+REPCRITS[10],
         "SettledAmount" : "Specify Settled Amount limits.  Now: "+REPCRITS[12],
         "Quit"          : "Return to the previous menu"
      default KCHOICE
      to KCHOICE

      switch
         case KCHOICE = "FilingAmount" :
            DATE_OR_NUMBER ( "Value", "Filing Amount", "Filing Amount", 10 )

         case KCHOICE = "SettledAmount" :
            DATE_OR_NUMBER ( "Value", "Settled Amount", "Settled Amount", 12 )

         otherwise :
            return

      endswitch
   endwhile

endproc

writelib LIBNAME AMOUNTS_CRITERIA
release procs AMOUNTS_CRITERIA
?? "."



; -------------------------------------------------------------------
; Process dates or numbers for one of the report criteria
; -------------------------------------------------------------------
proc DATE_OR_NUMBER ( WHICHTYPE,       ; text for prompts "Date" or "Value"
                      QUERYFIELD,      ; field being specified
                      FIELDTEXT,       ; text to describe the field
                      ELEMENT          ; which array element to set
                    )

   private KCHOICE,                    ; menu choice
           BEGVAL,                     ; beginning or only value
           ENDVAL,                     ; ending value
           A,                          ; temp variable
           B,                          ; temp variable
           PROCNAME                    ; procedure name

   PROCNAME = "PROCESS_DATE_NUMBER"
   BEGVAL   = ""
   ENDVAL   = ""

   moveto field QUERYFIELD
   switch
      case match([],"blank") :
         KCHOICE = "No"+WHICHTYPE
      case match([],">=.., <=..", A, B ) :
         KCHOICE  = "Between"
         BEGVAL   = A
         ENDVAL   = B
      case match([],">=..", A ) :
         KCHOICE = "GreaterThan"
         BEGVAL  = A
      case match([],"<=..", A ) :
         KCHOICE = "LessThan"
         BEGVAL  = A
      case not isblank([]) :
         KCHOICE = "One"+WHICHTYPE
         BEGVAL  = []
      otherwise :
         KCHOICE = "All"+WHICHTYPE+"s"
   endswitch

   while true
      showmenu
         "All"+WHICHTYPE+"s" : "Do not limit the search based on "+FIELDTEXT,
         "GreaterThan"       : "Find records greater than or equal to the specified "+FIELDTEXT,
         "LessThan"          : "Find records less than or equal to the specified "+FIELDTEXT,
         "Between"           : "Find records within a range of "+FIELDTEXT+"s",
         "One"+WHICHTYPE     : "Find records with only one "+FIELDTEXT,
         "No"+WHICHTYPE      : "Find records with a blank "+FIELDTEXT,
         "Quit"              : "Return to the previous menu"
      default KCHOICE
      to KCHOICE

      switch
         case KCHOICE = "All"+WHICHTYPE+"s" :
            ctrlbackspace
            REPCRITS [ ELEMENT ] = "All "+WHICHTYPE+"s used"

         case KCHOICE = "GreaterThan" :
            BEGVAL = ENTER_VALUE( "minimum "+lower(WHICHTYPE), WHICHTYPE, "", BEGVAL )
            if BEGVAL = False or isblank(BEGVAL) then
               loop
            endif
            ctrlbackspace
            typein ">=" + strval(BEGVAL)
            REPCRITS [ ELEMENT ] = ">= "+strval(BEGVAL)

         case KCHOICE = "LessThan" :
            BEGVAL = ENTER_VALUE( "maximum "+lower(WHICHTYPE), WHICHTYPE, "", BEGVAL )
            if BEGVAL = False or isblank(BEGVAL) then
               loop
            endif
            ctrlbackspace
            typein "<=" + strval(BEGVAL)
            REPCRITS [ ELEMENT ] = "<= "+strval(BEGVAL)

         case KCHOICE = "Between" :
            BEGVAL = ENTER_VALUE( "starting "+lower(WHICHTYPE), WHICHTYPE, "", BEGVAL )
            if BEGVAL = False or isblank(BEGVAL) then
               loop
            endif
            while (true)
               ENDVAL = ENTER_VALUE( "ending "+lower(WHICHTYPE), WHICHTYPE, "", ENDVAL )
               if ENDVAL = False or isblank(ENDVAL) then
                  quitloop
               endif
               if ENDVAL <= BEGVAL then
                  message "Ending value must be greater than Starting value; press a key"
                  retval = getchar()
               else
                  quitloop
               endif
            endwhile
            if ENDVAL = False then
               loop
            else
               ctrlbackspace
               typein ">="+strval(BEGVAL)+", <="+strval(ENDVAL)
               REPCRITS [ ELEMENT ] = ">="+strval(BEGVAL)+", <="+strval(ENDVAL)
            endif

         case KCHOICE = "One"+WHICHTYPE :
            BEGVAL = ENTER_VALUE( "exact "+lower(WHICHTYPE), WHICHTYPE, "", BEGVAL )
            if BEGVAL = False or isblank(BEGVAL) then
               loop
            endif
            ctrlbackspace
            typein strval(BEGVAL)
            REPCRITS [ ELEMENT ] = strval(BEGVAL)

         case KCHOICE = "No"+WHICHTYPE :
            ctrlbackspace
            typein "blank"
            REPCRITS [ ELEMENT ] = "No "+WHICHTYPE+" entered"

      endswitch
      return

   endwhile

endproc

writelib LIBNAME DATE_OR_NUMBER
release procs DATE_OR_NUMBER
?? "."



; -------------------------------------------------------------------
; Specify a value to be entered into a query field
; -------------------------------------------------------------------
proc ENTER_VALUE ( WHICHTEXT ,         ; text to present to the user
                   WHICHPROMPT ,       ; prompt for top line
                   DATAPIC ,           ; picture to use
                   WHICHVAL            ; default value
                  )

   private WHICHVAL,
           PROCNAME

   PROCNAME = "ENTER_VALUE"

   paintcanvas fill " " attribute 63 0,0,1,79
   style attribute 63
   @ 1,0 ?? "Specify the " + WHICHTEXT + " to use for this report"
   @ 0,0 ?? WHICHPROMPT + ": "
   cursor normal
   if DATAPIC = "" then
      accept "A20" default WHICHVAL to WHICHVAL
   else
      accept "A20" picture DATAPIC default WHICHVAL to WHICHVAL
   endif
   cursor off
   paintcanvas fill " " attribute 63 0,0,1,79
   if retval = False then
      return False
   else
      return WHICHVAL
   endif

endproc

writelib LIBNAME ENTER_VALUE
release procs ENTER_VALUE
?? "."


; -------------------------------------------------------------------
; Lookup a value from a table
; -------------------------------------------------------------------
proc LOOKUP_TABLE ( LUTABLE ,          ; table to use for lookup
                    LUFIELD ,          ; field to grab
                    QUERYFIELD ,       ; field in query image
                    ELEMENT            ; which array element to set
                  )

   private KCHOICE,
           PROCNAME

   PROCNAME = "LOOKUP_TABLE"

   view LUTABLE
   firstshow
   moveto field LUFIELD
   while (true)
      wait table
         prompt "Move the cursor to highlight the "+LUFIELD,
                "[F2] Select      [Esc] Remove value        [Ctrl-Z] Search    [Alt-Z] SearchNext"
      until "F2", "Esc", "DOS", "DOSBig", "Rotate", "FieldView", "F35"
      switch
         case retval = "F2" :
            moveto field LUFIELD
            KCHOICE = []
            clearimage
            moveto field QUERYFIELD
            ctrlbackspace
            typein KCHOICE
            REPCRITS [ ELEMENT ] = KCHOICE
            clear
            return True

         case retval = "Esc" :
            clearimage
            moveto field QUERYFIELD
            ctrlbackspace
            REPCRITS [ ELEMENT ] = "All values used"
            clear
            return False

         otherwise :
            beep

      endswitch

   endwhile

endproc

writelib LIBNAME LOOKUP_TABLE
release procs LOOKUP_TABLE
?? "."


release vars LIBNAME
reset

; --------------------------------------------------- end of script